@@ -34,7 +34,15 @@ class JSGenerator(NodeVisitor):
         return 'throw %s;' % self.visit(node.type)
 
     def visit_ExceptHandler(self, node):
-        return '\n'.join(map(self.visit, node.body))
+        out = ''
+        if node.type:
+            out = 'if (__exception__ == %s || isinstance([__exception__, %s])) {\n' % (self.visit(node.type), self.visit(node.type))
+        if node.name:
+            out += 'var %s = __exception__;\n' % self.visit(node.name)
+        out += '\n'.join(map(self.visit, node.body)) + '\n'
+        if node.type:
+            out += '}\n'
+        return out
 
     def visit_FunctionDef(self, node):
         args = self.visit(node.args)
@@ -113,7 +121,7 @@ class JSGenerator(NodeVisitor):
         else:
             if node.args:
                 args = [self.visit(e) for e in node.args]
-                args = ', '.join(args)
+                args = ', '.join([e for e in args if e])
             else:
                 args = ''
             return '%s(%s)' % (name, args)
@@ -198,7 +206,7 @@ class JSGenerator(NodeVisitor):
         test = self.visit(node.test)
         body = '\n'.join(map(self.visit, node.body)) + '\n'
         orelse = '\n'.join(map(self.visit, node.orelse)) + '\n'
-        if orelse:
+        if not orelse.isspace():
             return 'if(%s) {\n%s}\nelse {\n%s}\n' % (test, body, orelse)
         return 'if(%s) {\n%s}\n' % (test, body)
 
@@ -208,9 +216,9 @@ class JSGenerator(NodeVisitor):
         # iter is the python iterator
         init_iter = 'var iter = %s;\n' % iter
         # backup iterator and affect value of the next element to the target
-        pre = 'var backup = %s;\ni = iter[%s];\n' % (target, target)
+        pre = 'var backup = %s;\n%s = iter[%s];\n' % (target, target, target)
         # replace the replace target with the javascript iterator
-        post = 'i = backup;\n'
+        post = '%s = backup;\n' % target
         body = '\n'.join(map(self.visit, node.body)) + '\n'
         body = pre + body + post
         for_block = init_iter + 'for (var %s=0; %s < iter.length; %s++) {\n%s}\n' % (target, target, target, body)
